#include <cppshell/utils/signals.hpp>

#include <csignal>
#include <cerrno>

#if !defined(__APPLE__)
thread_local SigpipeState global_state;
#endif

bool IgnoreSigpipe() {
#if !defined(__APPLE__)
  sigset_t pending;
  sigemptyset(&pending);
  sigpending(&pending);
  bool sigpipe_pending = sigismember(&pending, SIGPIPE);

  if (!sigpipe_pending) {
    sigset_t sigpipe_mask;
    sigemptyset(&sigpipe_mask);
    sigaddset(&sigpipe_mask, SIGPIPE);

    sigset_t blocked;
    sigemptyset(&blocked);
    pthread_sigmask(SIG_BLOCK, &sigpipe_mask, &blocked);

    global_state.sigpipe_unblock = !sigismember(&blocked, SIGPIPE);

    return true;
  }
  return false;
#endif
  return false;
}

void UnignoreSigpipe() {
#if !defined(__APPLE__)
  sigset_t pending;
  sigemptyset(&pending);
  sigpending(&pending);
  bool sigpipe_pending = sigismember(&pending, SIGPIPE);

  sigset_t sigpipe_mask;
  sigemptyset(&sigpipe_mask);
  sigaddset(&sigpipe_mask, SIGPIPE);

  if (sigpipe_pending) {
    const struct timespec nowait = {0, 0};

    while (sigtimedwait(&sigpipe_mask, NULL, &nowait) == -1 && errno == EINTR) {
    }
  }

  if (global_state.sigpipe_unblock) {
    pthread_sigmask(SIG_UNBLOCK, &sigpipe_mask, NULL);
  }
#endif
}
